"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parser = exports.transformAST = void 0;
const wrapAstTransformation_1 = __importDefault(require("../src/wrapAstTransformation"));
const report_1 = require("../src/report");
const transformAST = ({ root, j }) => {
    var _a;
    // find the export default
    const defaultExportBody = root.find(j.ExportDefaultDeclaration);
    // stats
    const cntFunc = report_1.getCntFunc('add-emit-declarations', global.outputReport);
    // find the CallExpression
    const emitCalls = defaultExportBody.find(j.CallExpression, node => {
        var _a, _b;
        return (((_a = node.callee.object) === null || _a === void 0 ? void 0 : _a.type) === 'ThisExpression' &&
            ((_b = node.callee.property) === null || _b === void 0 ? void 0 : _b.name) === '$emit');
    });
    if (emitCalls.length) {
        // find the $emit argument
        const eventNames = [];
        emitCalls.forEach(({ node }) => {
            var _a;
            if (((_a = node.arguments[0]) === null || _a === void 0 ? void 0 : _a.type) === 'StringLiteral') {
                eventNames.push(node.arguments[0].value);
            }
        });
        // find the emit property
        const emitsProperty = defaultExportBody.find(j.ObjectProperty, node => {
            return node.key.name === 'emits' && node.value.type === 'ArrayExpression';
        });
        const elements = emitsProperty.length
            ? emitsProperty.get(0).node.value.elements
            : [];
        const emits = elements.map((r) => r.value);
        const hasEmitsProperty = emitsProperty.length;
        // push not declare event name into emits
        eventNames.forEach(r => {
            if (!emits.includes(r)) {
                elements.push(j.stringLiteral(r));
            }
        });
        if (!hasEmitsProperty) {
            // no emits property then create emits:[...]  AST
            (_a = defaultExportBody
                .get(0)
                .node.declaration.properties) === null || _a === void 0 ? void 0 : _a.unshift(j.objectProperty(j.identifier('emits'), j.arrayExpression(elements)));
            cntFunc();
        }
    }
};
exports.transformAST = transformAST;
exports.default = wrapAstTransformation_1.default(exports.transformAST);
exports.parser = 'babylon';
