"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parser = exports.transformAST = void 0;
const wrapAstTransformation_1 = __importDefault(require("../src/wrapAstTransformation"));
const add_import_1 = require("./add-import");
const report_1 = require("../src/report");
const transformAST = context => {
    const { root, j } = context;
    const cntFunc = report_1.getCntFunc('render-to-resolveComponent', global.outputReport);
    // find render function
    const renderCollections = root
        .find(j.ObjectMethod, node => {
        return node.key.name === 'render' && node.params.length === 1;
    })
        .filter(nodePath => nodePath.parent.parent.node.type === 'ExportDefaultDeclaration');
    if (!renderCollections.length)
        return;
    // add import
    add_import_1.transformAST(context, {
        specifier: { type: 'named', imported: 'resolveComponent' },
        source: 'vue'
    });
    renderCollections.forEach(({ node }) => {
        // @ts-ignore
        const paramName = node.params[0].name;
        // remove render function param
        node.params = [];
        const callExpressionCollection = j(node).find(j.CallExpression, node => {
            return node.callee.name === paramName && node.arguments.length === 1;
        });
        if (!callExpressionCollection.length)
            return;
        cntFunc();
        // find the component name
        const componentName = callExpressionCollection.get(0).node.arguments[0].value;
        // remove non-letter for complying variable name rules
        const componentVariableName = removeNonLetter(componentName);
        callExpressionCollection
            .get(0)
            .parent.insertBefore(j(`const ${componentVariableName} = resolveComponent('${componentName}')`)
            .find(j.VariableDeclaration)
            .get().node);
        // replace h('xxx') with resolveComponent('xxx')
        callExpressionCollection.replaceWith(
        // @ts-ignore
        nodePath => (nodePath.node.callee.name = componentVariableName));
    });
};
exports.transformAST = transformAST;
/**
 * remove non-letter and uppercase the first letter after non-letter
 * button-component => buttonComponent
 * @param str
 */
function removeNonLetter(str) {
    if (str) {
        let returnValue = '';
        for (let i = 0; i < str.length; i++) {
            // letter
            if ((str[i] >= 'a' && str[i] <= 'z') ||
                (str[i] >= 'A' && str[i] <= 'Z')) {
                returnValue += str[i];
            }
            else {
                // non-letter , remove and uppercase the first letter after non-letter
                i++;
                if (str[i] >= 'a' && str[i] <= 'z') {
                    returnValue += String.fromCharCode(str[i].charCodeAt(0) - 32);
                }
            }
        }
        return returnValue;
    }
}
exports.default = wrapAstTransformation_1.default(exports.transformAST);
exports.parser = 'babylon';
