"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parser = exports.transformAST = void 0;
const wrapAstTransformation_1 = __importDefault(require("../src/wrapAstTransformation"));
const add_import_1 = require("./add-import");
const report_1 = require("../src/report");
/**
 * import VueI18n from 'vue-i18n' -> import { createI18n } from 'vue-i18n'
 * new VueI18n() -> createI18n()
 * @param context
 */
const transformAST = context => {
    const { root, j } = context;
    const cntFunc = report_1.getCntFunc('vue-i18n-v9', global.outputReport);
    // find import
    const vueI18nImportDecls = root.find(j.ImportDeclaration, {
        source: {
            value: 'vue-i18n'
        }
    });
    const importedVueI18n = vueI18nImportDecls.find(j.ImportDefaultSpecifier);
    if (!importedVueI18n.length) {
        return;
    }
    // might use alias when import, get the real import name
    const localVueI18n = importedVueI18n.get(0).node.local.name;
    // import VueI18n from 'vue-i18n' -> import { createI18n } from 'vue-i18n'
    if (vueI18nImportDecls) {
        cntFunc();
        vueI18nImportDecls.remove();
        add_import_1.transformAST(context, {
            specifier: { type: 'named', imported: 'createI18n' },
            source: 'vue-i18n'
        });
    }
    // new VueI18n() -> createI18n()
    const newVueI18n = root.find(j.NewExpression, {
        callee: {
            type: 'Identifier',
            name: localVueI18n
        }
    });
    if (newVueI18n) {
        newVueI18n.replaceWith(({ node }) => {
            return j.callExpression(j.identifier('createI18n'), node.arguments);
        });
    }
};
exports.transformAST = transformAST;
exports.default = wrapAstTransformation_1.default(exports.transformAST);
exports.parser = 'babylon';
