"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parser = exports.transformAST = void 0;
const wrapAstTransformation_1 = __importDefault(require("../src/wrapAstTransformation"));
const add_import_1 = require("./add-import");
const remove_extraneous_import_1 = require("./remove-extraneous-import");
const report_1 = require("../src/report");
const report_2 = require("../src/report");
// new Router() -> createRouter()
const transformAST = context => {
    const { root, j, filename } = context;
    const routerImportDecls = root.find(j.ImportDeclaration, {
        source: {
            value: 'vue-router'
        }
    });
    // stats
    const cntFunc = report_1.getCntFunc('vue-router-v4', global.outputReport);
    const importedVueRouter = routerImportDecls.find(j.ImportDefaultSpecifier);
    if (importedVueRouter.length) {
        const localVueRouter = importedVueRouter.get(0).node.local.name;
        cntFunc();
        const newVueRouter = root.find(j.NewExpression, {
            callee: {
                type: 'Identifier',
                name: localVueRouter
            }
        });
        let localCreateRouter = 'createRouter';
        // find whether createRouter has been used
        const createRouterIdentifiers = root.find(j.Identifier, {
            name: 'createRouter'
        });
        const importedCreateRouter = importedVueRouter.find(j.ImportSpecifier, {
            imported: {
                name: 'createRouter'
            }
        });
        if (!importedCreateRouter.length) {
            if (createRouterIdentifiers.length) {
                // rename createRouter to newCreateRouter
                localCreateRouter = 'newCreateRouter';
                add_import_1.transformAST(context, {
                    specifier: {
                        type: 'named',
                        imported: 'createRouter',
                        local: localCreateRouter
                    },
                    source: 'vue-router'
                });
            }
            else {
                add_import_1.transformAST(context, {
                    specifier: { type: 'named', imported: 'createRouter' },
                    source: 'vue-router'
                });
            }
        }
        // filter the node to the manual list
        let ifEnd = false;
        newVueRouter.forEach(({ node }) => {
            if (node.arguments.length > 0 &&
                !j.ObjectExpression.check(node.arguments[0])) {
                const path = filename;
                const name = 'vue-router new VueRouter';
                const suggest = `Currently, only object expressions passed to \`new VueRouter\` can be transformed.`;
                const website = 'https://next.router.vuejs.org/guide/migration/index.html#new-router-becomes-createrouter';
                report_2.pushManualList(path, node, name, suggest, website);
                ifEnd = true;
                return;
            }
            if (node.arguments.length > 0) {
                // @ts-ignore
                const routerConfig = node.arguments[0];
                routerConfig.properties.forEach(p => {
                    if ((j.ObjectProperty.check(p) || j.Property.check(p)) &&
                        p.key.name === 'mode') {
                        const mode = p.value.value;
                        if (mode !== 'hash' && mode !== 'history' && mode !== 'abstract') {
                            const path = filename;
                            const name = 'vue-router mode';
                            const suggest = `mode must be one of 'hash', 'history', or 'abstract'`;
                            const website = 'https://next.router.vuejs.org/guide/migration/index.html#new-history-option-to-replace-mode';
                            report_2.pushManualList(path, p, name, suggest, website);
                            ifEnd = true;
                            return;
                        }
                    }
                });
            }
        });
        if (ifEnd) {
            return;
        }
        newVueRouter.replaceWith(({ node }) => {
            // mode: 'history' -> history: createWebHistory(), etc
            let historyMode = 'createWebHashHistory';
            let baseValue;
            if (!j.ObjectExpression.check(node.arguments[0])) {
                throw new Error('Currently, only object expressions passed to `new VueRouter` can be transformed.');
            }
            const routerConfig = node.arguments[0];
            routerConfig.properties = routerConfig.properties.filter(p => {
                if (!j.ObjectProperty.check(p) && !j.Property.check(p)) {
                    return true;
                }
                if (p.key.name === 'mode') {
                    const mode = p.value.value;
                    if (mode === 'hash') {
                        historyMode = 'createWebHashHistory';
                    }
                    else if (mode === 'history') {
                        historyMode = 'createWebHistory';
                    }
                    else if (mode === 'abstract') {
                        historyMode = 'createMemoryHistory';
                    }
                    else {
                        throw new Error(`mode must be one of 'hash', 'history', or 'abstract'`);
                    }
                    return false;
                }
                else if (p.key.name === 'base') {
                    baseValue = p.value;
                    return false;
                }
                else if (p.key.name === 'fallback') {
                    return false;
                }
                return true;
            });
            // add the default mode with a hash history
            add_import_1.transformAST(context, {
                specifier: { type: 'named', imported: historyMode },
                source: 'vue-router'
            });
            node.arguments[0].properties = node.arguments[0].properties.filter(p => !!p);
            node.arguments[0].properties.unshift(j.objectProperty(j.identifier('history'), j.callExpression(j.identifier(historyMode), baseValue ? [baseValue] : [])));
            return j.callExpression(j.identifier(localCreateRouter), node.arguments);
        });
        // VueRouter.START_LOCATION => import {START_LOCATION} from 'vue-router'
        const startLocationMember = root.find(j.MemberExpression, {
            object: {
                type: 'Identifier',
                name: localVueRouter
            },
            property: {
                type: 'Identifier',
                name: 'START_LOCATION'
            }
        });
        if (startLocationMember.length) {
            add_import_1.transformAST(context, {
                specifier: { type: 'named', imported: 'START_LOCATION' },
                source: 'vue-router'
            });
            startLocationMember.replaceWith(({ node }) => {
                return node.property;
            });
        }
        remove_extraneous_import_1.transformAST(context, {
            localBinding: localVueRouter
        });
    }
};
exports.transformAST = transformAST;
exports.default = wrapAstTransformation_1.default(exports.transformAST);
exports.parser = 'babylon';
